/*
  Copyright (C) 2004-2005 SKYRIX Software AG

  This file is part of OpenGroupware.org.

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/

#ifndef __GDLContentStore_GCSFolder_H__
#define __GDLContentStore_GCSFolder_H__

#import <Foundation/NSObject.h>

@class NSString, NSURL, NSNumber, NSArray, NSException, NSMutableString;
@class NSDictionary;
@class EOQualifier, EOFetchSpecification;
@class EOAdaptorChannel;
@class GCSFolderManager, GCSFolderType, GCSChannelManager;

@interface GCSFolder : NSObject
{
  GCSFolderManager *folderManager;
  GCSFolderType    *folderInfo;
  
  NSNumber *folderId;
  NSString *folderName;
  NSString *path;
  NSURL    *location;
  NSURL    *quickLocation;
  NSString *folderTypeName;

  struct {
    int requiresFolderSelect:1;
    int sameTableForQuick:1;
    int reserved:30;
  } ofFlags;
}

- (id)initWithPath:(NSString *)_path primaryKey:(id)_folderId
  folderTypeName:(NSString *)_ftname folderType:(GCSFolderType *)_ftype
  location:(NSURL *)_loc quickLocation:(NSURL *)_qloc
  folderManager:(GCSFolderManager *)_fm;

/* accessors */

- (NSNumber *)folderId;
- (NSString *)folderName;
- (NSString *)path;
- (NSURL    *)location;
- (NSURL    *)quickLocation;
- (NSString *)folderTypeName;

- (GCSFolderManager *)folderManager;
- (GCSChannelManager *)channelManager;

- (NSString *)storeTableName;
- (NSString *)quickTableName;
- (BOOL)isQuickInfoStoredInContentTable;

/* connection */

- (EOAdaptorChannel *)acquireStoreChannel;
- (EOAdaptorChannel *)acquireQuickChannel;
- (void)releaseChannel:(EOAdaptorChannel *)_channel;

- (BOOL)canConnectStore;
- (BOOL)canConnectQuick;

/* operations */

- (NSArray *)subFolderNames;
- (NSArray *)allSubFolderNames;

- (NSString *)fetchContentWithName:(NSString *)_name;
- (NSException *)writeContent:(NSString *)_content toName:(NSString *)_name;
- (NSException *)deleteContentWithName:(NSString *)_name;

- (NSDictionary *)fetchContentsOfAllFiles;

- (NSArray *)fetchFields:(NSArray *)_flds 
  fetchSpecification:(EOFetchSpecification *)_fs;
- (NSArray *)fetchFields:(NSArray *)_flds matchingQualifier:(EOQualifier *)_q;

@end

#endif /* __GDLContentStore_GCSFolder_H__ */
